package com.fr.plugin.jsd3998.sso;

import com.fr.base.PropertiesUtils;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.decision.webservice.exception.user.UserNotExistException;
import com.fr.decision.webservice.utils.DecisionServiceConstants;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.stable.StringUtils;
import com.fr.third.org.apache.commons.lang3.math.NumberUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Calendar;
import java.util.Properties;

// @EnableMetrics
public class H5SsoHandler extends BaseHttpHandler {

    @Override
    public RequestMethod getMethod() {
        return RequestMethod.GET;
    }

    @Override
    public String getPath() {
        return "/h5/callback";
    }

    @Override
    public boolean isPublic() {
        return true;
    }


    @Override
    public void handle(HttpServletRequest request, HttpServletResponse response) throws Exception {
        Properties props = PropertiesUtils.getProperties("sso");
        String encryptKey = props.getProperty("encrypt.key");
        Long ticketTimeoutInSeconds = NumberUtils.createLong(props.getProperty("ticket.timeoutInSeconds", "300"));
        String ticket = request.getParameter("ticket");
        FineLoggerFactory.getLogger().info("获取到的ticket值为[{}]", ticket);
        if (StringUtils.isNotBlank(ticket)) {
            ticket = DesECBUtil.decryptDES(ticket, encryptKey);
            FineLoggerFactory.getLogger().info("解密后的ticket值为[{}]", ticket);
            String[] tickets = ticket.split("\\+");
            String username = tickets[0];
            FineLoggerFactory.getLogger().info("获取到username为[{}]", username);
            Long ticketTime = NumberUtils.createLong(tickets[1]);
            Long nowTime = Calendar.getInstance().getTimeInMillis();
            JSONObject res = new JSONObject();
            if ((nowTime - ticketTime) > ticketTimeoutInSeconds * 1000) {
                FineLoggerFactory.getLogger().info("ticket[{}]已过期", ticket);
                request.getRequestDispatcher("/login.html").forward(request, response);
                res.put("res", "ticket已过期");
                WebUtils.printAsJSON(response, res);
                return;
            }
            if (loginFromToken(request, response, username)) {
                request.getRequestDispatcher( "/decision/url/mobile").forward(request, response);
            } else {
                res.put("res", "fail");
                WebUtils.printAsJSON(response, res);
            }
        } else {
            FineLoggerFactory.getLogger().info("未找到用户名，跳转到登录页面");
            request.getRequestDispatcher("/login.html").forward(request, response);
        }
    }

    protected boolean loginFromToken(HttpServletRequest req, HttpServletResponse res, String username) throws Exception {
        try {
            if (com.fr.stable.StringUtils.isNotEmpty(username)) {
                FineLoggerFactory.getLogger().info("current username:" + username);
                User user = UserService.getInstance().getUserByUserName(username);
                FineLoggerFactory.getLogger().info("get user:" + user);
                if (user == null) {
                    throw new UserNotExistException();
                }
                String token = LoginService.getInstance().login(req, res, username);
                FineLoggerFactory.getLogger().info("get login token:" + token);
                req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, token);
                FineLoggerFactory.getLogger().info("username:" + username + "  login success");
                return true;
            } else {
                FineLoggerFactory.getLogger().warn("username is null!");
                return false;
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return false;
    }

//    public static void main(String... args) throws Exception {
//        String ticket = URLDecoder.decode("7O%2Fyt%2FGeLqg8fxjY4Lm8%2F31FkSgLZh3a", "utf-8");
//        System.out.println(ticket);
//        ticket = DesECBUtil.decryptDES(ticket, "1111111111111111111");
//        System.out.println(ticket);
//        String[] tickets = ticket.split("\\+");
//        String username = tickets[0];
//        System.out.println(username);
//
//
//        String t = "1+" + Calendar.getInstance().getTimeInMillis();
//        System.out.println(URLEncoder.encode(DesECBUtil.encryptDES(t, "1111111111111111111"), "utf-8"));
//    }
}
