package com.fr.plugin.jsd3998.sso;

import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.exception.user.UserNotExistException;
import com.fr.decision.webservice.utils.DecisionServiceConstants;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.StringUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author luomiao
 * @since 2020/08/28
 */
@EnableMetrics
public class SsoHttpHandler extends BaseHttpHandler {
    @Override
    public RequestMethod getMethod() {
        return RequestMethod.GET;
    }

    @Override
    public String getPath() {
        return "/sso";
    }

    @Override
    public boolean isPublic() {
        return true;
    }

    @Override
    public void handle(HttpServletRequest request, HttpServletResponse response) throws Exception {
        String src = request.getParameter("src");
        String username = request.getParameter("username");
        if ("sys".equals(src) && StringUtils.isNotBlank(username)) {
            JSONObject res = new JSONObject();
            response.setContentType("application/json;charset=UTF-8");
            if (loginFromToken(request, response, username)) {
                res.put("res", "success");
                WebUtils.printAsJSON(response, res);
            } else {
                res.put("res", "fail");
                WebUtils.printAsJSON(response, res);
            }
        } else if (!"sys".equals(src)) {
            FineLoggerFactory.getLogger().info("未找到用户名，跳转到登录页面");
            request.getRequestDispatcher("/login.html").forward(request, response);
        }
    }

    protected boolean loginFromToken(HttpServletRequest req, HttpServletResponse res, String username) throws Exception {
        try {
            if (com.fr.stable.StringUtils.isNotEmpty(username)) {
                FineLoggerFactory.getLogger().info("current username:" + username);
                User user = UserService.getInstance().getUserByUserName(username);
                FineLoggerFactory.getLogger().info("get user:" + user);
                if (user == null) {
                    throw new UserNotExistException();
                }
                String token = LoginService.getInstance().login(req, res, username);
                FineLoggerFactory.getLogger().info("get login token:" + token);
                req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, token);
                FineLoggerFactory.getLogger().info("username:" + username + "  login success");
                return true;
            } else {
                FineLoggerFactory.getLogger().warn("username is null!");
                return false;
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return false;
    }


    protected void sendError(HttpServletResponse response, String errorCode) {
        Response res = Response.error(errorCode, errorCode);
        try {
            response.setContentType("application/json;charset=UTF-8");
            WebUtils.printAsJSON(response, JSONObject.mapFrom(res));
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("输出响应错误失败", e);
        }
    }
}
